export let diagramFilt = {"plugin": "peek_plugin_diagram"}
export let diagramTuplePrefix = "peek_plugin_diagram."

export let diagramObservableName = "peek_plugin_diagram"
export let diagramActionProcessorName = "peek_plugin_diagram"
export let diagramTupleOfflineServiceName = "peek_plugin_diagram"

export let diagramBaseUrl = "peek_plugin_diagram"

export let gridCacheStorageName = "peek_plugin_diagram_grids"
export let branchIndexStorageName = "peek_plugin_diagram_branch_index"
export let locationIndexCacheStorageName = "peek_plugin_diagram_location_index"

export let branchLocalStorageName = "peek_plugin_diagram_branch_local"

export let diagramPluginName = "peek_plugin_diagram"