"""Remove unique constraint for DispGroup

Peek Plugin Database Migration Script

Revision ID: 6f545d4166dc
Revises: 6a71c73c0606
Create Date: 2019-06-03 23:15:20.007675

"""

# revision identifiers, used by Alembic.
revision = "6f545d4166dc"
down_revision = "6a71c73c0606"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(
        "DispGroup_name_key", "DispGroup", schema="pl_diagram", type_="unique"
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(
        "DispGroup_name_key", "DispGroup", ["name"], schema="pl_diagram"
    )
    # ### end Alembic commands ###
